/**************************************************
 *	
 *	Name:	Pancake.java
 *	Author: Bill Fahle
 *	Date:	Aug '05
 *	Version: .81
 *	Comments: 
 *		improved file I/O by Bill - Jan '06 
 *		changes/comments by Walter Voit - Feb '06 
 * 
 *************************************************/
//package pancakes;


//package Pancakes;
import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.io.*;


public class PancakeGUI extends Applet implements Runnable, WindowListener
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	static PancakeGUI pan;
	
	private PancakeControls pan1,pan2;
	private DepthControl depthc;
	private FileControl filec;
	private TextArea ta;
	private Thread runner;
	private Panel pNorth, pSouth;
	private String[] args = new String[2];
	
	private boolean filestart = false;
	Pancake pancake;	
	
	public static void main(String[] args)
	{
		Frame f = new Frame("Flipping Pancakes");
		PancakeGUI pan = new PancakeGUI();
		pan.init();
		f.add("Center", pan);
		f.pack();
		f.setVisible(true);
		f.addWindowListener(pan);
		
	}

	// Implement WindowListener interface
	// to detect window closing
	public void windowActivated(WindowEvent e){}
	public void windowClosed(WindowEvent e){}
	public void windowClosing(WindowEvent e){stop();destroy();}
	public void windowDeactivated(WindowEvent e){}
	public void windowDeiconified(WindowEvent e){}
	public void windowIconified(WindowEvent e){}
	public void windowOpened(WindowEvent e){}

	public void filestart()
	{
		filestart = true;
		runner = new Thread(this);
		runner.start();
	}

	public void start()
	{
		filestart = false;
		runner = new Thread(this);
		runner.start();
	}
	public void stop() 
	{
		runner = null;
		super.stop();
		//		System.exit(0);
	}

	public void destroy() 
	{
	
		remove(pNorth);
		remove(pSouth);
		System.exit(0);
	}

	public void init()
	{
		pancake = new Pancake();
//		setBackground(Color.orange);
		setLayout(new BorderLayout());
		pNorth = new Panel();
		pSouth = new Panel();
		pSouth.setLayout(new GridLayout(4,1));
		add("North", pNorth);
		add("South", pSouth);
		pan1 = new PancakeControls(this, pancake.inputString, true);
		pan2 = new PancakeControls(this, pancake.inputControl , false);
		depthc = new DepthControl(this, "");
		filec = new FileControl(this, pancake.outputString);
		
		depthc.addExitButton();
		pan2.addRenderButton();
		pan1.addFileButton();
		ta = new TextArea("", 13, 40);
		pNorth.add(ta);
		pSouth.add(pan1);
		pSouth.add(pan2);
//		pSouth.add(sym);
		pSouth.add(depthc);
		pSouth.add(filec);
		add("North", pNorth);
		add("South", pSouth);
		ta.setText("");
		pancake.setTextArea(ta);
	}

	public void run() 
	{	    
		ta.setText("");
		pancake.setTextArea(ta);
		if (filestart)
		{
			try 
			{
				String file = pan1.getParam();
				FileParser fp = new FileParser(file);
				while(fp.pancakeMore())
				{
					if (fp.newcase())
						pancake.lineOut(fp.gpcase());
					String str = fp.gpcase();
					String gpcase = "";
					for (int i=0; i<str.length(); i++)
						if (str.charAt(i) != '-')
							gpcase += str.substring(i, i+1);
					pancake.lineOut(fp.casenum());
					args[0] = fp.objects();
					args[1] = gpcase;
					pancake.begin(args);
					pancake.lineOut("");
				}
			}
			catch(FileNotFoundException fnf)
			{
				pancake.lineOut("File not found: " + fnf.getMessage());
			}
		}
		else
		{
			args[0] = pan1.getParam();
			args[1] = pan2.getParam();
			pancake.depthControl = depthc.getParam();
			pancake.depthSubtractor = 0;
			pancake.findBest = filec.getCheckBoxStatus();
			pancake.outputString = filec.getParam();
			pancake.begin(args);
		}
	}

	public String getAppletInfo() 
	{
		return "An interactive program to run pancake flipping.";
	}

	public String[][] getParameterInfo() 
	{
		String[][] info = 
			{
				{"Objects", "b Aa Bc",
					"Abstract representation of flippable objects."},
				{"G+P Case", "3", 
					"Digit representation of successive Gates and Papadimitriou cases"}
			};
		return info;
	}

}


class PancakeControls extends Panel implements ActionListener 
{

	private static final long serialVersionUID = 1L;
	private CardinalTextField start;
	private Button button;
	private Button fbutton;
	private PancakeGUI applet;

	private static LayoutManager dcLayout = new FlowLayout(FlowLayout.RIGHT,
		10, 5);

	public PancakeControls(PancakeGUI app, String s, /* int e, int type, */
		boolean vertical) 
	{
		applet = app;
		setLayout(dcLayout);
		add(new Label(vertical ? "Objects" : "G+P Case"));
		add(start = new CardinalTextField(s, 34));
		//		add(end = new CardinalTextField(Integer.toString(e), 4));
	}

	/* puts on the button */
	public void addRenderButton() 
	{
		add(button = new Button("Flip"));
		button.addActionListener(this);
	}

	/* puts on the button */
	public void addFileButton() 
	{
		add(fbutton = new Button("File"));
		fbutton.addActionListener(this);
	}

	/* retrieves data from the user input fields */
	public String getParam() 
	{
		String s;
		try 
		{
			s = start.getText();
		} 
		catch (NumberFormatException nfe) 
		{
			s = null;
		}
		return s;
	}

	/* called when user clicks the button */
	public void actionPerformed(ActionEvent e) 
	{
		if (e.getSource() == button) 
		{
			applet.start();
		}
		else if (e.getSource() == fbutton)
		{
			applet.filestart();
		}
	}
}
class FileControl extends Panel
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private CardinalTextField filename;
	private Checkbox bestSol;

	private static LayoutManager dcLayout = new FlowLayout(FlowLayout.RIGHT,
		10, 5);

	public FileControl(PancakeGUI app, String s) 
	{
		setLayout(dcLayout);
		add(new Label("Output File"));
		add(filename = new CardinalTextField(s, 15));
		add(new Label("Find Best Solution?"));
		add(bestSol = new Checkbox());
		bestSol.setState(false);
	}
	
	public boolean getCheckBoxStatus()
	{
		return bestSol.getState();
	}


	/* retrieves data from the user input fields */
	public String getParam() 
	{
		return filename.getText();
	}
//	public void setParam(String s)
//	{
//		filename.setText(s);
//	}
	
}

class DepthControl extends Panel implements ActionListener 
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private CardinalTextField depth;
	private Button exuent;
	private PancakeGUI applet;

	private static LayoutManager dcLayout = new FlowLayout(FlowLayout.RIGHT,
		10, 5);

	public DepthControl(PancakeGUI app, String s) 
	{
		applet = app;
		setLayout(dcLayout);
		add(new Label("Total Steps"));
		add(depth = new CardinalTextField(s, 34));
	}

	/* puts on the button */
	public void addExitButton() 
	{
		add(exuent = new Button("Exit"));
		exuent.addActionListener(this);
	}

	/* retrieves data from the user input fields */
	public int getParam() 
	{
		int s;
		try 
		{
			s = Integer.parseInt(depth.getText());
		} 
		catch (NumberFormatException nfe) 
		{
			s = 0;
		}
		return s;
	}

	/* called when user clicks the button */
	public void actionPerformed(ActionEvent e) 
	{
		if (e.getSource() == exuent)
		{
			applet.stop();
			applet.destroy();
		}
	}
}

class CardinalTextField extends TextField 
{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	String oldText = null;

	public CardinalTextField(String text, int columns) 
	{ 
		super(text, columns);
		enableEvents(AWTEvent.KEY_EVENT_MASK | AWTEvent.TEXT_EVENT_MASK);
		oldText = getText();
	}

	// Consume non-digit KeyTyped events
	// Note that processTextEvent kind of eliminates the need for this 
	// function, but this is neater, since ideally, it would prevent 
	// the text from appearing at all.  Sigh.  See bugid 4100317/4114565. 
	//
	protected void processEvent(AWTEvent evt) 
	{
		int id = evt.getID();
		if (id != KeyEvent.KEY_TYPED) 
		{
			super.processEvent(evt);
			return;
		}

//		KeyEvent kevt = (KeyEvent) evt;
//		char c = kevt.getKeyChar();

		// Digits, backspace, and delete are okay 
		// Note that the minus sign is not allowed (neither is decimal)
//		if (Character.isDigit(c) || (c == '\b') || (c == '\u007f') 
//			|| (c == '\u0003') || (c == '\u0016')) 
//		{
			super.processEvent(evt);
			return;
//		}

//		Toolkit.getDefaultToolkit().beep();
//		kevt.consume();
	}

	// Should consume TextEvents for non-integer Strings
	// Store away the text in the tf for every TextEvent
	// so we can revert to it on a TextEvent (paste, or 
	// legal key in the wrong location) with bad text
	//
	// Note: it would be easy to extend this to an eight-bit 
	// TextField (range 0-255), but I'll leave it as-is.  
	//
	protected void processTextEvent(TextEvent te) 
	{ 
		// The empty string is okay, too 
		String newText = getText();
		if (newText.equals("") || textIsCardinal(newText)) 
		{
			oldText = newText;
			super.processTextEvent(te);
			return;
		}

		Toolkit.getDefaultToolkit().beep();
		setText(oldText);
	}

	// Returns true for Cardinal (non-negative) numbers
	// Note that the empty string is not allowed
	private boolean textIsCardinal(String textToCheck) 
	{
		if (textToCheck.length() > 0)
			return true;
		return false; 
	}
}
