/**************************************************
 *	
 *	Name:	Pancake.java
 *	Author: Walter Voit & Bill Fahle
 *	Date:	Feb '06
 *	Version: .81
 *	Comments: 
 *		object class to keep track of a permutation
 * 
 *************************************************/
//package pancakes;


class Perm
{
	char[] perm;	//the input string that we are manipulating
	int block;		// number of blocks
	int sing;		// number of singletons
	int flip;		// number of flips executed
	int adjacencies; // number of adjacencies max to form (each postFlip success will form one).
	public static char BLOCKCHAR = '~'; //represents how the rest of the block is symbolized
	public static int FLIP_COEF = 11; //20; 
	public static int BLOCK_COEF = 24; //39;
	public static int SING_COEF = 18; // 39;
//	public static int FLIP_COEF = 135;
//	public static int BLOCK_COEF = 295;
//	public static int SING_COEF = 221;
	public Perm parent;
	
	//new constructor given an input string
	Perm(String str, int adj)
	{
		perm = new char[str.length()];
		for (int i=0; i<perm.length; i++)
		{
			perm[i] = str.charAt(i);
			if (perm[i] == '|' || perm[i] == '*' || perm[i] == '.')
				perm[i] = ' '; // in case input was hastily pasted (say that 3 times fast)
		}
		block = 0;
		flip = 0;
		sing = 0;
		adjacencies = adj;
		parent = null;
	}

	//copy constructor
	Perm(Perm rhs)
	{
		parent = rhs;
		perm = new char[rhs.perm.length];
		for (int i=0; i<rhs.perm.length; i++)
			perm[i] = rhs.perm[i];
		block = rhs.block;
		flip = rhs.flip;
		sing = rhs.sing;
		adjacencies = rhs.adjacencies;
		
	}

	//returns the length of the string
	int length()
	{
		return perm.length;
	}

	//flips the first 'n' chars in the perm string
	void flip(int n)
	{
		char[] ret = new char[this.perm.length];
		for (int i=0; i<n; i++)
		{
			ret[i] = this.perm[n-1-i];
		}
		for (int i=n; i<this.perm.length; i++)
		{
			ret[i] = this.perm[i];
		}
		this.perm = ret;
		this.flip++;
	}

	//tests whether the flip was legal
	boolean legalFlip(int n)
	{
		if (n >= perm.length)
			return false;
		if (perm[n] != ' ')
			return false; // can't break groups already formed	
		Perm temp = this.parent;
		while (temp != null)
		{
			//have we seen this string before
			if (this.perm.equals(temp.perm))
				return false;
			else
				temp = temp.parent;			
		}
		
		return true;
	}

	//helper method to determine whether an input char is a 
	//lower case letter (i.e. a singleton or part of a block)
	private boolean isALetter(char symbol)
	{
		if ((symbol >= 'A' && symbol <='Z')||(symbol >= 'a' && symbol <='z'))
			return true;
		return false;
	}

	//helper method to see if letters are consecutive
	private boolean areSuccessive(char first, char second)
	{
		if ( (first == second+1) || (first+1 == second) )
			return true;
		if (( first == 'Z'&& second == 'a')||
				( first == 'a'&& second == 'Z'))
			return true;
		
		return false;
	}

	//method to combine neighboring singletons into blocks and
	//add singleton-block neighbors into a single block
	//returns true if we fixed spacing issues
	boolean postFlip(int x)
	{
		if (perm[x] != ' ')
			return false;
		// cases we want are:
		//		1 	_ Ac d _
		//		2	_ Ac dE _
		//		3	_ Ad c _
		// 		4	_ Ad cE _
		//		5	_ Ad c
		//		6	_ Ac d
		//		7	 _ c d _
		//		8	 _ d c _
		//		9	 _ c dE _
		//		10	 _ d cA _
		// where x is here:         ^

		// first check for cases 7 and 8.
		if (x>0 && x<perm.length-1 && (x==1 || perm[x-2] == ' ')
			&& (x==perm.length-2 || perm[x+2] == ' '))
		{
			if (isALetter(perm[x-1]))
			{  
				if (areSuccessive(perm[x-1], perm[x+1]))
				{
					perm[x] = BLOCKCHAR; // new group
					block++;
					sing -= 2;
					adjacencies--;
					return true;
				}
			} 
			//			return false;
		}

		// check for cases 9 and 10.
		if (x>0 && x<perm.length-2 && (x==1 || perm[x-2] == ' ')
			&& (x<perm.length-1 && perm[x+1] != ' '))
		{
			if (isALetter(perm[x-1]))
			{  
				if (areSuccessive(perm[x-1], perm[x+1]))
				{
					char[] ret = new char[perm.length-2];
					for (int i=0; i<x; i++)
						ret[i] = perm[i];
					for (int i=x+2; i<perm.length; i++)
						ret[i-2] = perm[i];
					sing--;
					perm = ret;
					adjacencies--;
					return true;
				}
			} 
			//			return false;
		}
		// cases 2 and 4
		if (x>1 && x<perm.length-2 && perm[x-2] != ' ' &&
			perm[x+2] != ' ')
		{
			if (isALetter(perm[x-1]))
			{  
				if (areSuccessive(perm[x-1], perm[x+1]))
				{
					char[] ret = new char[perm.length-4];
					for (int i=0; i<x-1; i++)
						ret[i] = perm[i];
					for (int i=x+3; i<perm.length; i++)
						ret[i-4] = perm[i];
					block--;
					perm = ret;
					adjacencies--;
					return true;
				}
			} 
			//			return false;
		}
		// remaining cases:

		if (x>0 && x<perm.length-1)
		{
			if (isALetter(perm[x-1]))
			{
				if (areSuccessive(perm[x-1], perm[x+1]))
				{ 
					// concat Ba b... to Bb or Bb a to Ba
					char[] ret = new char[perm.length-2];
					for (int i=0; i<x-1; i++)
						ret[i] = perm[i];
					for (int i=x+1; i<perm.length; i++)
						ret[i-2] = perm[i];
					sing--;
					perm = ret;
					adjacencies--;
					return true;
				}
			}
		}
		return false;
	}

	//methods to give access to private member variables
	char[] ch()			{return perm;}
	int sing()			{return sing;}
	int block()			{return block;}
	int flip()			{return flip;}
	int adjacencies()	{return adjacencies;}

    //returns G&P bound score
	int oldscore()
	{
		// best score is lowest - given in thirds, 0 = G&P
		return flip*3 + block*7 + sing*5;
	}

	//returns our improved score
	int score()
	{
		return flip*FLIP_COEF + block*BLOCK_COEF + sing*SING_COEF;
	}
}
